//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <stdlib.h>
#include <allegro.h>
#include "building.h"
#include "images.h"
#include "frame_play.h"
#include "player.h"
#include "soldier_ai.h"


///////////////
// Constructor
Building::Building() {
	HasOwner = false;
	Heals = false;
	NoShoot = false;
	Position = 0;
	type = 3;
}


//////////////
// Destructor
Building::~Building() {
	Soldier *cur;
	for (size_t i=0;i<positions.size();i++) {
		cur = positions[i].soldier;
		if (cur) LeaveBuilding(cur);
	}
}


//////////
// Update
void Building::Update() {
	// Check if it's too far
	if (ABS(playFrame->Player->x - x) > 4000.0) {
		//Destroy();
		return;
	}
/*
	// Delete any clones of it
	Building *curBuilding;
	std::list<Sprite*>::iterator parent_end = parent->Sprites.end();
	for (std::list<Sprite*>::iterator cur=parent->Sprites.begin();cur!=parent_end;cur++) {
	    Sprite* sp;
	    if ((sp = *cur) != this && sp->type == 3) {
			curBuilding = (Building*) (sp);
			if (curBuilding->x == x) curBuilding->Destroy();
		}
	}
*/
	// Update as object
	Object::Update();
}


////////
// Draw
void Building::Draw() {
	// Draw soldiers
	for (size_t i=0;i<positions.size();i++) {
		if (positions[i].soldier) positions[i].soldier->Draw();
	}

	// Draw as object
	Object::Draw();
}


//////////////////////////////////////
// Count number of units within range
int Building::CountUnitsWithinRange(int distance,bool enemy) {
	int n = 0;
	Soldier *current;
	if (enemy) {
		for (std::list<Soldier*>::iterator cur=playFrame->SoldierEnemy.begin();cur!=playFrame->SoldierEnemy.end();cur++) {
			current = *cur;
			if (current->Alive && ABS(current->x-x) < distance) n++;
		}
	}
	else {
		for (std::list<Soldier*>::iterator cur=playFrame->SoldierFriendly.begin();cur!=playFrame->SoldierFriendly.end();cur++) {
			current = *cur;
			if (current->Alive && ABS(current->x-x) < distance) n++;
		}
	}
	return n;
}


///////////////////
// Capture attempt
void Building::CaptureAttempt(Soldier *soldier) {
	if ((!HasOwner || soldier->IsEnemy != IsEnemy) && soldier->CollidesWith(this)) {
		bool isEmpty = true;
		for (size_t i=0;i<positions.size();i++) {
			if (positions[i].soldier) {
				isEmpty = false;
				break;
			}
		}
		if (isEmpty) {
			IsEnemy = soldier->IsEnemy;
			if (!IsEnemy) playFrame->Player->CapturedPosition(int(x/(50*32)+0.5),soldier == playFrame->Player);
			Captured(soldier);
		}
	}
}


////////////////////////////
// Get position to enter on
int Building::GetFreePosition() {
	// Find position
	for (size_t i=0;i<positions.size();i++) {
		if (positions[i].soldier == NULL) {
			return i;
		}
	}
	return -1;
}


//////////////////
// Enter building
void Building::EnterBuilding(Soldier *soldier) {
	// Same type?
	if (HasOwner && soldier->IsEnemy != IsEnemy) return;

	// Already on a building?
	if (soldier->Inside) return;

	// Found a vacant slot
	int pos = GetFreePosition();
	if (pos != -1) {
		positions.at(pos).soldier = soldier;
		soldier->x = x+positions[pos].x;
		soldier->y = y+positions[pos].y;
		soldier->vx = 0;
		soldier->vy = 0;
		soldier->GravityEffect = 0;
		soldier->Hidden = true;
		soldier->Inside = this;
		soldier->SetAnimSequence(0);
		soldier->DamageMultiply = positions.at(pos).DamageMultiply;
		if (soldier->IsAI) {
			SoldierAI *ai = (SoldierAI*) soldier;
			ai->LookForTarget();
		}
	}

	// Force generation of surrounding flagpoles
	int n = playFrame->CellAtPos(x);
	int cell_n;
	for (int i=-2;i<=2;i++) {
		cell_n = n+i*50;
		Building *building = playFrame->GetTerrain(cell_n)->building;
		if (building) {
			building->x = cell_n*32;
			building->y = playFrame->HeightAtPos(cell_n*32);
			playFrame->AddSprite(building);
			playFrame->GetTerrain(cell_n)->building = NULL;
		}
	}
}


//////////////////
// Leave building
void Building::LeaveBuilding(Soldier *soldier) {
	// Find and remove
	size_t pos = FindSoldier(soldier);
	positions.at(pos).soldier = NULL;
	soldier->GravityEffect = 1.0;
	soldier->Hidden = false;
	soldier->Inside = NULL;
	soldier->DamageMultiply = 1;
}



////////////////
// Find soldier
int Building::FindSoldier(Soldier *soldier) {
	int pos = -1;
	int n = 0;
	for (size_t i=0;i<positions.size();i++) {
		if (positions[i].soldier == soldier) {
			pos = i;
			n++;
		}
	}
	if (n != 1) {
		throw "Soldier doesn't exist!";
	}
	return pos;
}


////////////////////////
// Flagpole constructor
BuildingFlagpole::BuildingFlagpole(bool _IsEnemy) {
	// Setup vars
	HasOwner = true;
	InitialSoldiers = false;
	Heals = true;
	bmp = Images.GetImage("flagpole.bmp");
	pivotX = 32;
	pivotY = 100;
	step = 0;
	timer = 10;
	IsEnemy = _IsEnemy;
	BuildingType = 1;

	// Setup flag
	flag = new Sprite();
	if (IsEnemy) flag->bmp = Images.GetImage("enemy_flag.bmp");
	else flag->bmp = Images.GetImage("friendly_flag.bmp");
	flag->pivotX = 0;
	flag->pivotY = 5;

	// Setup light
	light = new Sprite();
	light->bmp = Images.GetImage("flagpole_light.bmp");
	light->pivotX = 0;
	light->pivotY = 0;

	// Setup positions
	BuildingPosition pos;
	pos.DamageMultiply = 0.5;
	pos.x = -12;
	pos.y = -30;
	positions.push_back(pos);
	pos.x = 12;
	positions.push_back(pos);
}


///////////////////////
// Flagpole destructor
BuildingFlagpole::~BuildingFlagpole() {
}


/////////////////
// Flagpole draw
void BuildingFlagpole::Draw() {
	// Draw flag
	flag->x = x+2;
	if (step == 1 || step == 2) flag->y = y-85+timer;
	else flag->y = y-85;
	flag->Draw();

	// Draw building
	Building::Draw();

	// Draw light
	if ((step == 0 && timer < 80) || step == 3) {
		light->x = x-6;
		light->y = y-22;
		light->Draw();
	}
}


///////////////////
// Flagpole update
void BuildingFlagpole::Update() {
	// Spawn initial soldiers
	if (!InitialSoldiers) {
		// Soldiers
		for (int i=0;i<4;i++) {
			playFrame->AddSoldier(new SoldierAI((rand()%640)-320 + x,IsEnemy,int(MAX(1,ABS(x/1600)))));
		}
		InitialSoldiers = true;
	}

	// Waiting to spawn
	if (step == 0) {
		timer--;
		if (timer == 0) {
			step = 3;
		}
	}

	// Taking flag down
	if (step == 1) {
		timer++;
		if (timer == 80) {
			if (IsEnemy) flag->bmp = Images.GetImage("enemy_flag.bmp");
			else flag->bmp = Images.GetImage("friendly_flag.bmp");
			step = 2;
		}
	}

	// Taking flag up
	else if (step == 2) {
		timer--;
		if (timer == 0) {
			step = 0;
			timer = (rand()%200)+100;
		}
	}

	// Spawning
	if (step == 3) {
		if (ABS(playFrame->Player->x - x) < 3000) {
			timer++;
			if (timer == 120) {
				step = 0;
				if (CountUnitsWithinRange(320,IsEnemy) < 5) timer = (rand()%100)+100;
				else timer = (rand()%300)+500;
			}
			if (rand() % 7 == 0) {
				if (CountUnitsWithinRange(320,IsEnemy) < 10) {
					SpawnSoldier(true);
				}
			}
		}
	}

	// Heals
	if (playFrame->Player->Inside == this && timer % 10 == 0) {
		//playFrame->Player->Heal(1);
	}

	// Building
	Building::Update();
}


///////////////////////
// Flagpole set parent
void BuildingFlagpole::SetParent(FrameBase *parent) {
	// stuff
	playFrame = (FramePlay*) parent;
	flag->SetParent(parent);
	light->SetParent(parent);
	Sprite::SetParent(parent);
}


////////////
// Captured
void BuildingFlagpole::Captured(Soldier *soldier) {
	step = 1;
	timer = 0;
	if (!soldier->IsEnemy) {
		int last = playFrame->LastCaptured;
		int cur = int(ABS(x/1600)+0.5);
		if (cur > last) {
			playFrame->LastCaptured = cur;
			playFrame->TimeToCapture = 100*60;
		}
	}
}


/////////
// Spawn
void BuildingFlagpole::SpawnSoldier(bool update) {
	Soldier *soldier = new SoldierAI(x,IsEnemy,int(MAX(1,ABS(x/1600)+0.5)));
	playFrame->AddSoldier(soldier);
	if (GetFreePosition() != -1) EnterBuilding(soldier);
	if (update) soldier->Update();
}


/////////////////////////////////
// Building position constructor
BuildingPosition::BuildingPosition() {
	soldier = NULL;
}


///////////
// Sandbag
BuildingSandbag::BuildingSandbag() {
	// Setup vars
	bmp = Images.GetImage("sandbag.bmp");
	pivotX = 10;
	pivotY = 4;
	Position = 1;
	BuildingType = 2;

	// Setup positions
	BuildingPosition pos;
	pos.DamageMultiply = 0.5;
	pos.x = -3;
	pos.y = 0;
	positions.push_back(pos);
	pos.x = +3;
	positions.push_back(pos);
}


//////////
// Bunker
BuildingBunker::BuildingBunker() {
	// Setup vars
	bmp = Images.GetImage("bunker.bmp");
	pivotX = 32;
	pivotY = 32;
	Position = 0;
	NoShoot = true;
	BuildingType = 3;

	// Setup positions
	BuildingPosition pos;
	pos.DamageMultiply = 0.0;
	pos.x = 15;
	pos.y = -8;
	positions.push_back(pos);
	pos.x = -15;
	positions.push_back(pos);
}


///////////////
// Watch tower
BuildingWatchTower::BuildingWatchTower() {
	// Setup vars
	bmp = Images.GetImage("watch_tower.bmp");
	pivotX = 24;
	pivotY = 84;
	Position = 0;
	BuildingType = 4;

	// Setup positions
	BuildingPosition pos;
	pos.DamageMultiply = 0.7;
	pos.x = 12;
	pos.y = -78;
	positions.push_back(pos);
	pos.x = 0;
	positions.push_back(pos);
	pos.x = -12;
	positions.push_back(pos);
}
