//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//


#ifndef BUILDING_H
#define BUILDING_H


///////////
// Headers
#include <vector>
#include "object.h"


//////////////
// Prototypes
class Soldier;
class FramePlay;


/////////////////////
// Building position
class BuildingPosition {
public:
	BuildingPosition();

	Soldier* soldier;
	double x;
	double y;
	double DamageMultiply;
};


//////////////////
// Building class
class Building : public Object {
protected:
	std::vector<BuildingPosition> positions;
	virtual void Captured(Soldier *soldier) {}
	int GetFreePosition();

public:
	bool Heals;
	bool HasOwner;
	bool IsEnemy;
	bool NoShoot;
	int BuildingType;

	Building();
	virtual ~Building();

	virtual void Update();
	virtual void Draw();
	int CountUnitsWithinRange(int distance,bool IsEnemy);
	void CaptureAttempt(Soldier *soldier);
	void EnterBuilding(Soldier *soldier);
	void LeaveBuilding(Soldier *soldier);
	int FindSoldier(Soldier *soldier);
};


////////////
// Flagpole
class BuildingFlagpole : public Building {
private:
	int timer;
	int step;
	Sprite *flag;
	Sprite *light;
	bool InitialSoldiers;

	void Captured(Soldier *soldier);
	void SpawnSoldier(bool update=false);

public:
	BuildingFlagpole(bool IsEnemy);
	~BuildingFlagpole();

	void Draw();
	void Update();
	void SetParent(FrameBase *parent);
};


////////////
// Sandbags
class BuildingSandbag : public Building {
public:
	BuildingSandbag();
};


//////////
// Bunker
class BuildingBunker : public Building {
public:
	BuildingBunker();
};


//////////////
// Watch tower
class BuildingWatchTower : public Building {
public:
	BuildingWatchTower();
};

#endif
