//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <allegro.h>
#include <time.h>
#include <sys/mman.h>
#include "core.h"
#include "input.h"
#include "audio.h"
#include "frame_base.h"
#include "frame_play.h"
#include "frame_title.h"
#include "particles.h"
#include "animation.h"
#include "highscores.h"
#include <allegro/platform/aintlnx.h>


#define MESGDSTCTRL      0x0000
#define MESG_BSTBPP      0x60
#define MESG_DSTBPP_16  (1<<5)
#define MESG_DSTENB     (1<<6)
#define MESGDSTADDR      0x0004
#define MESGSRCCTRL      0x000c
#define MESG_INVIDEO    (1<<8)
#define MESGPATCTRL      0x0020
#define MESGSIZE         0x002c
#define MESG_HEIGHT      16
#define MESGCTRL         0x0030
#define MESG_XDIR_POS    (1<<8)
#define MESG_YDIR_POS    (1<<9)
#define MESG_FFCLR       (1<<10)
#define MESGSTATUS       0x0034
#define MESG_BUSY        (1<<0)


#define mmsp2_blit_putl(addr,value) (*(unsigned long volatile *)(mmsp2_blit_base+(addr)) = (value))
#define mmsp2_blit_getl(addr)       (*(unsigned long volatile *)(mmsp2_blit_base+(addr)))


///////////////
// Constructor
CoreClass::CoreClass() {
 	FullScreen = true;
//  	FullScreen = false;
	Run = true;
	ForceRedraw = true;
	CurrentFrame = NULL;
	steps = 0;
	FrameType = 2;
}


//////////////
// Initialize
void CoreClass::Init() {

    unsigned long gp2x_dev     = open("/dev/mem",   O_RDWR); 
    static volatile unsigned long *gp2x_memregl = 
	(unsigned long  *)mmap(0, 0x10000, 
			       PROT_READ|PROT_WRITE, MAP_SHARED,
			       gp2x_dev, 0xc0000000);
    volatile unsigned short *gp2x_memregs = (unsigned short *)gp2x_memregl;

    // the following is to work around bug in allegro library which causes
    // it to lock up if HW blitter hasn't done anything since GP2X was 
    // switched on.
    
    gp2x_memregs[0x90a >> 1] = 0xffff; // Enable all video and sound devices
    gp2x_memregs[0x904 >> 1] |= 1<<10; // Enable FASTIO for the hardware blitter
    static struct MAPPED_MEMORY mmsp2_blit;
    mmsp2_blit.base  = 0xe0020000;
    mmsp2_blit.size  = 0x100;
    mmsp2_blit.perms = PROT_READ|PROT_WRITE;
    mmsp2_blit.data  = mmap(0, 0x100, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev, 0xe0020000);
    
    if (mmsp2_blit.data == MAP_FAILED)
	exit(EXIT_FAILURE);
    
    int mmsp2_blit_base = (int)mmsp2_blit.data;
    
    mmsp2_blit_putl (MESGDSTCTRL, MESG_DSTBPP_16 | MESG_DSTENB);
    mmsp2_blit_putl (MESGDSTADDR, 0x3101000);
    mmsp2_blit_putl (MESGSRCCTRL, MESG_INVIDEO);
    mmsp2_blit_putl (MESGPATCTRL, 0);
    mmsp2_blit_putl (MESGSIZE, (1 << MESG_HEIGHT) | 8);
    mmsp2_blit_putl (MESGCTRL, (MESG_XDIR_POS) | (MESG_YDIR_POS) | 0xaa | MESG_FFCLR);
    mmsp2_blit_putl (MESGSTATUS, 1);
    
    while (mmsp2_blit_getl(MESGSTATUS) & MESG_BUSY);
    
    munmap((void *)&mmsp2_blit_base, 0x100);

    // End of work around

    // Init allegro
    allegro_init();
    install_timer();
    install_keyboard();
    install_joystick(JOY_TYPE_AUTODETECT);

    // Set graphics
    int mode;
    if (FullScreen) mode = GFX_AUTODETECT_FULLSCREEN;
    else mode = GFX_AUTODETECT_WINDOWED;
    set_color_depth(16);
    set_gfx_mode(mode,320,240,0,0);
    BackBuffer = create_bitmap(screen->w,screen->h);
    set_display_switch_mode(SWITCH_BACKGROUND);
    set_window_title("Bunkermaster - ArchMageZeratuL's Speedhack 05 Version");

    // Set audio
    install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL);
    Audio.Init();

    // Load stuff
    Particle::InitBlueprints();
    Animation::Init();
    HighScores.New();

    // Set frame
    //CurrentFrame = new FramePlay;
}


////////////////
// Deinitialize
void CoreClass::DeInit() {
	destroy_bitmap(BackBuffer);
	allegro_exit();
}


/////////
// Timer
volatile int Ticks = 0;
void DoTick () { 
	Ticks++;
}
END_OF_FUNCTION(DoTick)


/////////////
// Game loop
void CoreClass::RunGameLoop() {
	// Lock and init timer
	LOCK_VARIABLE(Tick);
    LOCK_FUNCTION(DoTick);
	install_int_ex(DoTick, BPS_TO_TIMER(60));

	// Set Run = false to quit
	bool refresh = false;
	int NoRefresh = 0;
	GetFrame();
	while (Run) {
		while (Ticks > 0 && Run) {
			if (NoRefresh > 5) {
				Ticks = 0;
			}
			else {
				steps++;
				Audio.Update();
				Input.Update();
				refresh = UpdateCurrentFrame();
				if (!FrameOn) {
					GetFrame();
					refresh = false;
				}
				else {
					Ticks--;
					NoRefresh++;
				}
			}
		}
		if (Run && refresh) {
			NoRefresh = 0;
			DrawCurrentFrame();
			DrawScreen();
			refresh = false;
		}
		else rest(0);
	}

	// Remove Allegro timer
	remove_int(DoTick);
}


////////////////////////////////////
// Marks proper rectangles as dirty
void CoreClass::MarkDirty(int x,int y,int w,int h) {
	// Calculate edges
	int rx1,ry1,rx2,ry2;
	rx1 = MIN(MAX(0,x/8),79);
	ry1 = MIN(MAX(0,y/8),59);
	rx2 = MIN(MAX(0,(x+w)/8),79);
	ry2 = MIN(MAX(0,(y+h)/8),59);

	// Set them to two
	for (int i=rx1;i<=rx2;i++) {
		for (int j=ry1;j<=ry2;j++) {
			DirtyRects[i][j] = 2;
		}
	}
}


///////////////
// Draw screen
void CoreClass::DrawScreen () {
	// Check if backbuffer exists
	if (!BackBuffer) return;

	// Lock surface
	int dx,dy;
	acquire_screen();
	//vsync();

	// Draw all
	if (ForceRedraw) {
		blit (BackBuffer,screen,0,0,0,0,screen->w,screen->h);
		ForceRedraw = false;
	}

	// Dirty rects
	else {
		for (int i=0;i<40;i++) {
			for (int j=0;j<30;j++) {
				if (DirtyRects[i][j] > 0) {
					dx = i*8;
					dy = j*8;
					DirtyRects[i][j]--;
					blit (BackBuffer,screen,dx,dy,dx,dy,8,8);
				}
			}
		}
	}

	// Unlock surface
	release_screen();
}


//////////////////////
// Draw current frame
void CoreClass::DrawCurrentFrame() {
	if (CurrentFrame) CurrentFrame->Draw();
}


//////////////////////
// Update current frame
bool CoreClass::UpdateCurrentFrame() {
	if (CurrentFrame) return CurrentFrame->Update();
	else return false;
}


//////////////////////
// Make absolute path
void CoreClass::MakeAbsolutePath(const char *name,char *path) {
	char temp[1024];
	get_executable_name(temp,512);
	*get_filename(temp) = 0;
	strcat(temp,name);
	strcpy(path,temp);
}


//////////////////
// Get next frame
void CoreClass::GetFrame() {
	if (CurrentFrame) delete CurrentFrame;
	CurrentFrame = NULL;

	if (FrameType == 0) Run = false;
	else {
		// Reseed
		Seed = time(NULL);
		srand(Seed);
		FrameOn = true;

		// Get frame
		if (FrameType == 1) CurrentFrame = new FramePlay();
		if (FrameType == 2) CurrentFrame = new FrameTitle();
	}
}


///////////////////
// Global instance
CoreClass Core;
