//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//


#ifndef FRAME_PLAY_H
#define FRAME_PLAY_H


///////////
// Headers
#include <list>
#include "frame_base.h"
#include "rotating_cache.h"


//////////////
// Prototypes
struct BITMAP;
class PlayerSoldier;
class Soldier;
class Building;


////////////////
// Terrain unit
class TerrainUnit {
private:
	double GetNoise(double x,double amplitude,double length,int seed);

public:
	TerrainUnit();
	TerrainUnit(int n);
	~TerrainUnit();

	Building *building;
	double height;
	void Generate(int n);
};


////////////////////
// Play field frame
class FramePlay : public FrameBase {
private:
	int CurScore;
	int CursorHold;
	int PlayerDeadCount;
	bool GetHigh;
	int MenuPos;
	char PlayerName[16];
	int NameCur;
	BITMAP *bg;

	RotatingCache<TerrainUnit*,4096> Terrain;

public:
	int LastCaptured;
	int TimeToCapture;

	bool Pause;
	static FramePlay *PlayFrame;
	PlayerSoldier *Player;
	std::list<Soldier*> SoldierEnemy;
	std::list<Soldier*> SoldierFriendly;

	FramePlay();
	~FramePlay();

	void DrawInterface();
	void DrawMinimap();
	bool Update();
	void Draw();
	int CellAtPos(double x);
	TerrainUnit *GetTerrain(int n);
	double HeightAtPos(double x);
	void DrawTerrain();
	void DrawBackground();
	void AddSoldier(Soldier* soldier);
	void DrawSoldiers();
	void UpdateSoldiers();
	void UpdateMenu();
	void DrawMenu();
	void GetHighscore();
};


#endif
