//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <allegro.h>
#include "object.h"
#include "frame_play.h"
#include "particles.h"
#include "audio.h"


///////////////
// Constructor
Object::Object() {
	vx = vy = 0;
	DieOnHit = false;
	GravityEffect = 1.0;
	GroundDrag = 0.0;
	BackOnHit = false;
	HeightSupport = 0;
	Bounce = 0;
	DragX = DragY = 0.99;
}


//////////////
// Set parent
void Object::SetParent(FrameBase *_parent) {
	playFrame = (FramePlay*) _parent;
	Sprite::SetParent(_parent);
}


//////////
// Update
void Object::Update() {
	// Gravity
	vy += 0.2*GravityEffect;

	// Movement
	x += vx;
	y += vy;

	// Drag
	vx *= DragX;
	vy *= DragY;

	// Collision
	double height = playFrame->HeightAtPos(x);
	if (y > height+HeightSupport) {
		if (DieOnHit) {
			Destroy();
			return;
		}
		if (BackOnHit) Position = 1;
		double Up = 0;
		if (GravityEffect < 0) Up = -1.5;
		int extra = int(y - (height+double(HeightSupport)));
		if (Bounce > 0) {
			y = height+HeightSupport+Up-(extra*Bounce);
			vy = -vy*Bounce;
		}
		else {
			y = height+HeightSupport+Up+0.5;
			vy = 0;
		}
		vx -= vx*GroundDrag;
	}

	// Update as sprite
	Sprite::Update();
}


////////////////////////////
// Check if it is on ground
bool Object::IsOnGround() {
	double height = playFrame->HeightAtPos(x);
	return (y+0.5 > height+HeightSupport);
}


//////////////////
// Spawn particle
Particle *Object::SpawnParticle(int n,int spawnX,int spawnY) {
	//if (!bmp) throw "Bitmap not defined";
	Particle *part = new Particle(n);
	part->SetParent(parent);
	parent->AddSprite(part);
	int dx;
	if (bmp && FlipHorizontal) dx = bmp->w - spawnX - pivotX;
	else dx = spawnX-pivotX;
	part->x = x+dx;
	part->y = y+spawnY-pivotY;
	part->parent = this;
	return part;
}


////////////////////////////
// Get pan for stereo audio
int Object::GetSoundPan() {
	return MID(0,int(x-parent->scrollX)*255/640,255);
}


////////////////////////////////////////////////
// Get volume for stereo audio (outside screen)
int Object::GetSoundVolume() {
        int vol = int(Audio.GetVolume() * 255.0f);
	int pos = int(x-parent->scrollX);
	if (pos < 0) return MID(0,vol+(pos),vol);
	if (pos > 640) return MID(0,vol-(pos-640),vol);
	return vol;
}


////////////////////////
// Checks for collision
bool Object::CollidesWith(Object *target) {
	// Check if it fails bounding box test
	int x1,x2,y1,y2,w1,w2,h1,h2;
	GetDrawXYWH(x1,y1,w1,h1);
	target->GetDrawXYWH(x2,y2,w2,h2);
	if (x1+w1 < x2 || x1 > x2+w2 || y1+h1 < y2 || y1 > y2 + h2) return false;

	// Collided
	return true;
}


//////////////
// Play sound
void Object::PlaySound(const char*name) {
	int vol = GetSoundVolume();
	if (vol) play_sample(Audio.GetSample(name),vol,GetSoundPan(),950+(rand()%101),0);
}
