//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <math.h>
#include <allegro.h>
#include "player.h"
#include "input.h"
#include "images.h"
#include "animation.h"
#include "building.h"


///////////////
// Constructor
PlayerSoldier::PlayerSoldier(double x) : Soldier(x,false) {
	speed = 3;
	MaxLife = 500;
	Direction = 1;
	Life = 500;
	CrossHair = new Sprite();
	CrossHair->bmp = Images.GetImage("cross.bmp");
	CrossHair->pivotX = 4;
	CrossHair->pivotY = 4;
	Score = 0;
}


//////////////
// Destructor
PlayerSoldier::~PlayerSoldier() {
	delete CrossHair;
}


//////////
// Update
void PlayerSoldier::Update() {
	// Dead?
	if (!Alive) {
		Stop();
		Soldier::Update();
		return;
	}

	// Aim
	if (Input.Key[Key_Up]) angle += 0.05;
	if (Input.Key[Key_Down]) angle -= 0.05;
	if (Weapon != 1) angle = MID(-0.5,angle,0.5);
	else angle = MID(-0.5,angle,1);

	// Movement
	bool OnGround = IsOnGround();
	if (Input.Key[Key_Left]) Move(-1);
	else if (Input.Key[Key_Right]) Move(1);
	else if (OnGround) Stop();

	// Jump
	//if (OnGround) {
	//	if (Input.KeyDown[Key_B]) Jump();
	//}

	// Shoot
	if (Input.Key[Key_A]) Shoot();

	// Buildings
	if (Input.KeyDown[Key_B]) {
		// Leave
		if (Inside) {
			Inside->LeaveBuilding(this);
		}

		else {
			// Enter/Capture
			Building *building = OverBuilding();
			if (building) {
				if (building->HasOwner && building->IsEnemy) Capture(building);
				else building->EnterBuilding(this);
			}
		}
	}

	// Change weapon
	if (Input.KeyDown[Key_Select]) NextWeapon();

	Soldier::Update();
}


////////
// Draw
void PlayerSoldier::Draw() {
	if (Alive) {
		CrossHair->SetParent(parent);
		CrossHair->x = x + Direction*cos(angle)*80.0;
		CrossHair->y = y - sin(angle)*80.0 - 5;
	}
	Soldier::Draw();
}


///////////////////////
// Captured a position
void PlayerSoldier::CapturedPosition(int n,bool byPlayer) {
	if (byPlayer) Score += 1000+n*100;
}
