//
// BunkerMaster - Copyright (c) 2005 Rodrigo Braz Monteiro
// Rodrigo Braz Monteiro's entry for Speedhack 05
// This source file is subject to GNU General Public License
// See included license.txt for more information
//
// You can reach the author at:
// http://zeratul.cellosoft.com
// zeratul@cellosoft.com
//
///////////
// Headers
#include <allegro.h>
#include "animation.h"
#include "sprite.h"
#include "frame_base.h"
#include "core.h"

#ifndef NO_FBLEND
#include <fblend.h>
#endif


///////////////
// Constructor
Sprite::Sprite() {
	drawMode = 0;
	bmp = NULL;
	Position = 2;
	FlipHorizontal = false;
	x = y = 0;
	parent = NULL;
	anim = NULL;
	anim_seq = 0;
	anim_time = 0;
	anim_frame = 0;
	type = 0;
	alpha = 255;
	Hidden = false;
}


//////////////
// Destructor
Sprite::~Sprite() {
}


////////
// Draw
void Sprite::Draw() {
	if (!bmp) return;
	int dx,dy,w,h;
	GetDrawXYWH(dx,dy,w,h);
	if (drawMode == 0) {
		if (FlipHorizontal) draw_sprite_h_flip(Core.BackBuffer,bmp,dx,dy);
		else draw_sprite(Core.BackBuffer,bmp,dx,dy);
	}
	if (drawMode == 1) {
		#ifndef NO_FBLEND
		fblend_add(bmp,Core.BackBuffer,dx,dy,alpha);
		#else
		set_add_blender(0,0,0,alpha);
		draw_trans_sprite(Core.BackBuffer,bmp,dx,dy);
		#endif
	}
	if (drawMode == 2) {
		#ifndef NO_FBLEND
		fblend_trans(bmp,Core.BackBuffer,dx,dy,alpha);
		#else
		set_trans_blender(0,0,0,alpha);
		draw_trans_sprite(Core.BackBuffer,bmp,dx,dy);
		#endif
	}
	Core.MarkDirty(dx,dy,bmp->w,bmp->h);
}


///////////////////////////
// Get drawing coordinates
void Sprite::GetDrawXYWH(int &dx,int &dy,int &dw,int &dh) {
	if (!bmp || !parent) {
		dx = dy = dw = dh = 0;
		return;
	}
	dx = int(x - parent->scrollX);
	if (FlipHorizontal) dx -= (bmp->w - pivotX);
	else dx -= pivotX;
	dy = int(y - parent->scrollY - pivotY);
	dw = bmp->w;
	dh = bmp->h;
}


//////////////
// Set parent
void Sprite::SetParent(FrameBase *_parent) {
	parent = _parent;
}


///////
// Die
void Sprite::Destroy() {
	parent->DestroySprite(this);
}


//////////
// Update
void Sprite::Update() {
	if (anim) {
		anim_time++;
		AnimationSequence *curSeq = anim->group[anim_seq];
		int frames = curSeq->seq.size();
		if (frames) {
			anim_frame = 0;
			int totalLen = 0;
			for (int i=0;i<frames;i++) {
				if (anim_time > totalLen) anim_frame = i;
				totalLen += curSeq->seq.at(i).len;
			}
			if (anim_time > totalLen) {
				anim_frame = 0;
				anim_time = 0;
				if (curSeq->NextSequence != -1) anim_seq = curSeq->NextSequence;
			}
			bmp = curSeq->seq.at(anim_frame).bmp;
			pivotX = curSeq->seq.at(anim_frame).pivotX;
			pivotY = curSeq->seq.at(anim_frame).pivotY;
		}
	}
}


//////////////////////////
// Set animation sequence
void Sprite::SetAnimSequence(int n) {
	if (anim_seq != n) {
		anim_seq = n;
		anim_frame = 0;
		anim_time = 0;
	}
}
